/*globals define, Image*/

define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/shop/ui/dashboard-notifications-test',
    'jquery',
    'de_epages/shop/ui/dashboard-notifications',

    '$ready!'
], function (QUnit, fixture, $, de_epages) {

    QUnit.module("de_epages/product/ui/variationuploadimages-unity", {

        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("widget test presence of methods", function () {

        $.each([
            '_create',
            'destroy'
        ], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.shopUiDashboardNotifications.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest("checking widget functionality (deleting single items)", function () {
        fixture.init();

        var selectors = {
                hiddenRowNumber: '#ep-dashboard-hidden-notification-number',
                showMoreButton:'#ep-dashboard-show-more-button',
                showMoreRow: '.ep-dashboard-show-more',
                emptyNotificationRow: '.ep-dashboard-notification-empty',
                notificationRow: '.ep-dashboard-notification',
                removeButton: '.ep-dashboard-cancel'
            },
            currentId;

        ep.ajax = function(data) {
            QUnit.equal(
                data.data.ChangeAction,
                'testChangeAction',
                'Ajax: Correct ChangeAction used.'
            );

            QUnit.equal(
                data.data.ObjectID,
                currentId,
                'Ajax: Correct ObjectId used.'
            );

            return {
                done: function(fn) {
                    fn();

                    return {
                        fail: function() {
                            return;
                        }
                    };
                }
            };
        };

        de_epages('.ep-dashboard-notification-container').shopUiDashboardNotifications({
          showMore: true,
          selectors: selectors,
          removeChangeAction: "testChangeAction"
        });


        window.setTimeout(function () {
            var widget = ep('.ep-dashboard-notification-container').data('uiShopUiDashboardNotifications'),
                deleteableRows = ep(selectors.notificationRow + ':has(' + selectors.removeButton + ')');

            QUnit.equal(
                (widget !== undefined),
                true,
                'widget is resolved'
            );

            QUnit.equal(
                ep(selectors.notificationRow + ':not(.HideElement)').length,
                4,
                'four notifications are visible'
            );

            currentId = deleteableRows.eq(0).data('notificationConfig').id;
            deleteableRows.eq(0).find(selectors.removeButton).trigger('click');

            window.setTimeout(function() {

                QUnit.equal(
                    ep(selectors.notificationRow + ':not(.HideElement)').length,
                    4,
                    'four notifications are visible after deleting one'
                );

                QUnit.ok(
                    deleteableRows.eq(0).hasClass('HideElement'),
                    'the deleted notification is hidden'
                );

                QUnit.equal(
                    parseInt(ep(selectors.hiddenRowNumber).text()),
                    2,
                    'the hidden notifications count is two'
                );

                currentId = deleteableRows.eq(1).data('notificationConfig').id;
                deleteableRows.eq(1).find(selectors.removeButton).trigger('click');

                window.setTimeout(function() {

                    QUnit.equal(
                        ep(selectors.notificationRow + ':not(.HideElement)').length,
                        5,
                        'five notifications are visible'
                    );

                    QUnit.ok(
                        deleteableRows.eq(1).hasClass('HideElement'),
                        'the deleted notification is hidden'
                    );

                    QUnit.equal(
                        ep(selectors.showMoreRow).length,
                        0,
                        'the show more row is removed'
                    );

                    fixture.destroy();
                    QUnit.start();
                }, 100);
            }, 100);
        }, 100);
    });

    QUnit.asyncTest("checking widget functionality (Show All)", function () {
        fixture.init();

        var selectors = {
                hiddenRowNumber: '#ep-dashboard-hidden-notification-number',
                showMoreButton:'#ep-dashboard-show-more-button',
                showMoreRow: '.ep-dashboard-show-more',
                notificationRow: '.ep-dashboard-notification',
                emptyNotificationRow: '.ep-dashboard-notification-empty',
                removeButton: '.ep-dashboard-cancel'
            };

        de_epages('.ep-dashboard-notification-container').shopUiDashboardNotifications({
          showMore: true,
          selectors: selectors,
          removeChangeAction: "testChangeAction"
        });

        QUnit.equal(
            ep(selectors.notificationRow + ':not(.HideElement)').length,
            4,
            'four notifications are visible'
        );

        ep(selectors.showMoreButton).trigger('click');

        QUnit.equal(
            ep(selectors.notificationRow + ':not(.HideElement)').length,
            7,
            'Seven notifications are visible after clicking show all'
        );

        fixture.destroy();
        QUnit.start();
    });

    QUnit.asyncTest("checking empty list functionality", function () {
        fixture.init();


        ep.ajax = function(data) {
            return {
                done: function(fn) {
                    fn();

                    return {
                        fail: function() {
                            return;
                        }
                    };
                }
            };
        };

        var selectors = {
                hiddenRowNumber: '#ep-dashboard-hidden-notification-number',
                showMoreButton:'#ep-dashboard-show-more-button',
                showMoreRow: '.ep-dashboard-show-more',
                notificationRow: '.ep-dashboard-notification',
                emptyNotificationRow: '.ep-dashboard-notification-empty',
                removeButton: '.ep-dashboard-cancel'
            },
            widgetElem = de_epages('.ep-dashboard-notification-container'),
            deleteButtons = ep(selectors.notificationRow + ' ' + selectors.removeButton);

        widgetElem.find(selectors.notificationRow).slice(1).remove();

        widgetElem.shopUiDashboardNotifications({
          showMore: true,
          selectors: selectors,
          removeChangeAction: "testChangeAction"
        });

        QUnit.equal(
            ep(selectors.notificationRow + ':not(.HideElement)').length,
            1,
            'one notification is visible'
        );

        deleteButtons.eq(0).trigger('click');

        QUnit.ok(
            !ep(selectors.emptyNotificationRow).hasClass('HideElement'),
            'empty notification is visible'
        );

        QUnit.ok(
            ep(selectors.notificationRow).hasClass('HideElement'),
            'notification are not visible'
        );

        fixture.destroy();
        QUnit.start();
    });
});